from typing import Dict

from msgspec import Struct


class PropertyType(Struct):
    type: str  # property type
    name: str  # property name
    field: str  # property field for affix
    affix: bool  # is relic or light cone affix
    ratio: bool  # is added ratio
    percent: bool  # is percent
    order: int  # property order
    icon: str  # property icon path


PropertyIndex = Dict[str, PropertyType]



from typing import Dict, List

from msgspec import Struct

from .common import Promotion, Property, Quantity


class CharacterEvaluationType(Struct):
    image: str
    link: str


class CharacterType(Struct):
    id: str
    name: str
    tag: str
    rarity: int
    path: str
    element: str
    max_sp: float
    ranks: List[str]
    skills: List[str]
    skill_trees: List[str]
    icon: str
    preview: str
    portrait: str
    guide_overview: List[str]
    guide_material: List[str]
    guide_evaluation: List[CharacterEvaluationType]


class CharacterRankType(Struct):
    id: str
    rank: int
    desc: str
    materials: List[Quantity]
    level_up_skills: List[Quantity]
    icon: str


class CharacterSkillType(Struct):
    id: str
    name: str
    max_level: int
    element: str
    type: str
    type_text: str
    effect: str
    effect_text: str
    simple_desc: str
    desc: str
    params: List[List[float]]
    icon: str


class SkillTreeLevelType(Struct):
    promotion: int
    properties: List[Property]
    materials: List[Quantity]


class CharacterSkillTreeType(Struct):
    id: str
    max_level: int
    anchor: str
    pre_points: List[str]
    level_up_skills: List[Quantity]
    levels: List[SkillTreeLevelType]
    icon: str


class CharacterPromotionType(Struct):
    id: str
    values: List[Dict[str, Promotion]]
    materials: List[List[Quantity]]


CharacterIndex = Dict[str, CharacterType]
CharacterRankIndex = Dict[str, CharacterRankType]
CharacterSkillIndex = Dict[str, CharacterSkillType]
CharacterSkillTreeIndex = Dict[str, CharacterSkillTreeType]
CharacterPromotionIndex = Dict[str, CharacterPromotionType]



from typing import List, Optional

from msgspec import Struct


class LevelInfo(Struct):
    id: str
    level: int = 0


class AvatarInfo(Struct):
    id: str
    name: str
    icon: str


class PathInfo(Struct):
    id: str
    name: str
    icon: str


class ElementInfo(Struct):
    id: str
    name: str
    color: str
    icon: str


class SkillInfo(Struct):
    id: str
    name: str
    level: int
    max_level: int
    element: Optional[ElementInfo]
    type: str
    type_text: str
    effect: str
    effect_text: str
    simple_desc: str
    desc: str
    icon: str


class SkillTreeInfo(Struct):
    id: str
    level: int
    icon: str


class PropertyInfo(Struct):
    type: str
    field: str
    name: str
    icon: str
    value: float
    display: str
    percent: bool


class AttributeInfo(Struct):
    field: str
    name: str
    icon: str
    value: float
    display: str
    percent: bool


class SubAffixInfo(Struct):
    id: str
    cnt: int
    step: int = 0


class RelicBasicInfo(Struct):
    id: str
    level: int = 1
    main_affix_id: Optional[str] = None
    sub_affix_info: List[SubAffixInfo] = []


class LightConeBasicInfo(Struct):
    id: str
    rank: int = 1
    level: int = 1
    promotion: int = 0


class CharacterBasicInfo(Struct):
    id: str
    rank: int = 0
    level: int = 1
    promotion: int = 0
    skill_tree_levels: List[LevelInfo] = []
    light_cone: Optional[LightConeBasicInfo] = None
    relics: Optional[List[RelicBasicInfo]] = None


class RelicInfo(Struct):
    id: str
    name: str
    set_id: str
    set_name: str
    rarity: int
    level: int
    icon: str
    main_affix: Optional[PropertyInfo] = None
    sub_affix: List[PropertyInfo] = []


class RelicSetInfo(Struct):
    id: str
    name: str
    icon: str
    num: int
    desc: str = ""
    properties: List[PropertyInfo] = []


class LightConeInfo(Struct):
    id: str
    name: str
    rarity: int
    rank: int
    level: int
    promotion: int
    icon: str
    preview: str
    portrait: str
    path: Optional[PathInfo] = None
    attributes: List[AttributeInfo] = []
    properties: List[PropertyInfo] = []


class CharacterInfo(Struct):
    id: str
    name: str
    rarity: int
    rank: int
    level: int
    promotion: int
    icon: str
    preview: str
    portrait: str
    rank_icons: List[str] = []
    path: Optional[PathInfo] = None
    element: Optional[ElementInfo] = None
    skills: List[SkillInfo] = []
    skill_trees: List[SkillTreeInfo] = []
    light_cone: Optional[LightConeInfo] = None
    relics: List[RelicInfo] = []
    relic_sets: List[RelicSetInfo] = []
    attributes: List[AttributeInfo] = []
    additions: List[AttributeInfo] = []
    properties: List[PropertyInfo] = []



from typing import Dict

from msgspec import Struct


class ElementType(Struct):
    id: str  # element id
    name: str  # element name
    desc: str  # element description
    color: str  # element color
    icon: str  # element icon path


ElementIndex = Dict[str, ElementType]
